/*
 * (C) Copyright 2000
 * Rob Taylor, Flying Pig Systems. robt@flyingpig.com.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */
 
#include "config.h"
#include "ns16550.h"
#include "ns87308.h"

/*
 * Minimal serial functions needed to use one87308's UARTs
 * as serial console interface.
 */

#if (CONFIG_CONS_INDEX == 1)
#define console (CFG_ISA_IO + COM1)
#elif (CONFIG_CONS_INDEX == 2)
#define console (CFG_ISA_IO + COM2)
#else
#error "Invalid COM port for console output (must be 1 or 2)."
#endif

void
serial_init (unsigned long dummy, int baudrate)
{
    int clock_divisor = 115200/baudrate;

    initialise_ns87308();
    
    NS16550_init((NS16550_t)console, clock_divisor);
}

void
serial_putc(const char c)
{
	if (c == '\n')
		NS16550_putc((NS16550_t)console, '\r');
		
    NS16550_putc((NS16550_t)console, c);
}

void
serial_puts (const char *s)
{
    while (*s) {
        serial_putc (*s++);
    }
}


int
serial_getc(void)
{
    return NS16550_getc((NS16550_t)console);
}

int
serial_tstc(void)
{
    return NS16550_tstc((NS16550_t)console);
}

void
serial_setbrg (unsigned long dummy, int baudrate)
{
   int clock_divisor = 115200/baudrate;
   NS16550_reinit((NS16550_t)console, clock_divisor);
}
