/*
 * (C) Copyright 2001
 * Stefan Roese, esd gmbh germany, stefan.roese@esd-electronics.com
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <ppcboot.h>
#include "cpci405.h"
#include <asm/processor.h>
#include <405gp_i2c.h>
#include <command.h>
#include <cmd_boot.h>

/* ------------------------------------------------------------------------- */

#if 0
#define FPGA_DEBUG
#endif

/* fpga configuration data - generated by bin2cc */
const unsigned char fpgadata[] =
{
#include "fpgadata.c"
};

/*
 * include common fpga code (for esd boards)
 */
#include "../common/fpga.c"


int board_pre_init (void)
{
  int index, len, i;
  int status;

#ifdef FPGA_DEBUG
  /* set up serial port with default baudrate */
  serial_init(0, CONFIG_BAUDRATE);
  console_init_f();
#endif

  /*
   * Boot onboard FPGA
   */
  status = fpga_boot_compressed();
  if (status != 0)
    {
      /* booting FPGA failed */
#ifndef FPGA_DEBUG
      /* set up serial port with default baudrate */
      serial_init(0, CONFIG_BAUDRATE);
      console_init_f();
#endif 
      printf("\nFPGA: Booting failed ");
      switch (status)
        {
        case ERROR_FPGA_PRG_INIT_LOW:
          printf("(Timeout: INIT not low after asserting PROGRAM*)\n ");
          break;
        case ERROR_FPGA_PRG_INIT_HIGH:
          printf("(Timeout: INIT not high after deasserting PROGRAM*)\n ");
          break;
        case ERROR_FPGA_PRG_DONE:
          printf("(Timeout: DONE not high after programming FPGA)\n ");
          break;
        }

      /* display infos on fpgaimage */
      index = 15;
      for (i=0; i<4; i++)
	{
	  len = fpgadata[index];
	  printf("FPGA: %s\n", &(fpgadata[index+1]));
	  index += len+3;
	}
      putc ('\n');  
      /* delayed reboot */
      for (i=20; i>0; i--)
	{
	  printf("Rebooting in %2d seconds \r",i);
	  for (index=0;index<1000;index++)
	    udelay(1000);
	}
      putc ('\n');  
      do_reset(NULL, NULL, 0, 0, NULL);
    }

  /*
   * IRQ 0-15  405GP internally generated; active high; level sensitive
   * IRQ 16    405GP internally generated; active low; level sensitive
   * IRQ 17-24 RESERVED
   * IRQ 25 (EXT IRQ 0) CAN0; active low; level sensitive
   * IRQ 26 (EXT IRQ 1) CAN1; active low; level sensitive
   * IRQ 27 (EXT IRQ 2) PCI SLOT 0; active low; level sensitive
   * IRQ 28 (EXT IRQ 3) PCI SLOT 1; active low; level sensitive
   * IRQ 29 (EXT IRQ 4) PCI SLOT 2; active low; level sensitive
   * IRQ 30 (EXT IRQ 5) PCI SLOT 3; active low; level sensitive
   * IRQ 31 (EXT IRQ 6) COMPACT FLASH; active high; level sensitive
   */
  mtdcr(uicsr, 0xFFFFFFFF);        /* clear all ints */
  mtdcr(uicer, 0x00000000);        /* disable all ints */
  mtdcr(uiccr, 0x00000000);        /* set all to be non-critical*/
  mtdcr(uicpr, 0xFFFFFF81);        /* set int polarities */
  mtdcr(uictr, 0x10000000);        /* set int trigger levels */
  mtdcr(uicvcr, 0x00000001);       /* set vect base=0,INT0 highest priority*/
  mtdcr(uicsr, 0xFFFFFFFF);        /* clear all ints */

  return 0;
}


/* ------------------------------------------------------------------------- */

int ctermm2(void)
{
  if ((*(unsigned char *)0xf0000400 == 0x00) &&
      (*(unsigned char *)0xf0000401 == 0x01))
    return 0;                           /* no, board is cpci405 */
  else
    return -1;                          /* yes, board is cterm-m2 */
}


int cpci405_host(void)
{
  if (mfdcr(strap) & PSR_PCI_ARBIT_EN)
    return -1;                          /* yes, board is cpci405 host */
  else
    return 0;                           /* no, board is cpci405 adapter */
}


int misc_init_f(void)
{
  return 0;  /* dummy implementation */
}


void misc_init_r(bd_t *bd)
{
  char *	tmp;                    /* Temporary char pointer      */

  /*
   * Generate last byte of ip-addr from code-plug @ 0xf0000400
   */
  if (ctermm2())
    {
      char str[32];
      unsigned char ipbyte = *(unsigned char *)0xf0000400;

      /*
       * Only overwrite ip-addr with allowed values
       */
      if ((ipbyte != 0x00) && (ipbyte != 0xff))
        {
          bd->bi_ip_addr = (bd->bi_ip_addr & 0xffffff00) | ipbyte;
          sprintf(str, "%ld.%ld.%ld.%ld",
                  (bd->bi_ip_addr & 0xff000000) >> 24,
                  (bd->bi_ip_addr & 0x00ff0000) >> 16,
                  (bd->bi_ip_addr & 0x0000ff00) >> 8,
                  (bd->bi_ip_addr & 0x000000ff));
          setenv("ipaddr", str);
        }
    }

  /*
   * Write ethernet addr in NVRAM for VxWorks
   */
  tmp = (char *)CFG_NVRAM_BASE_ADDR + CFG_NVRAM_VXWORKS_OFFS;
  memcpy( (char *)tmp, (char *)&bd->bi_enetaddr[0], 6 );
}


/*
 * Check Board Identity:
 */

int checkboard (void)
{
    int index;
    int len;
    unsigned char str[64];
    int i = getenv_r ("serial#", str, sizeof(str));

    if (i == -1) {
    	printf ("### No HW ID - assuming CPCI405");
    }
    else
      puts(str);

    if (ctermm2())
      printf(" (CTERM-M2 - Id=0x%02x)", *(unsigned char *)0xf0000400);
    else
      if (cpci405_host())
        printf(" (PCI Host Version)");
      else
        printf(" (PCI Adapter Version)");

    puts("\nFPGA:  ");

    /* display infos on fpgaimage */
    index = 15;
    for (i=0; i<4; i++)
      {
        len = fpgadata[index];
        printf("%s ", &(fpgadata[index+1]));
        index += len+3;
      }

    putc ('\n');

    return 1;
}

/* ------------------------------------------------------------------------- */

long int initdram (int board_type)
{
  unsigned long val;

  mtdcr(memcfga, mem_mb0cf);
  val = mfdcr(memcfgd);

#if 0
  printf("\nmb0cf=%x\n", val); /* test-only */
  printf("strap=%x\n", mfdcr(strap)); /* test-only */
#endif
  
  return (4*1024*1024 << ((val & 0x000e0000) >> 17));
}

/* ------------------------------------------------------------------------- */

int testdram (void)
{
    /* TODO: XXX XXX XXX */
    printf ("test: 16 MB - ok\n");

    return (0);
}

/* ------------------------------------------------------------------------- */
