/*
 * (C) Copyright 2000
 * Rob Taylor, Flying Pig Systems. robt@flyingpig.com.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <ppcboot.h>
#include "ns16550.h"

#define IDATA (init_data_t *)(CFG_INIT_RAM_ADDR + CFG_INIT_DATA_OFFSET)

void initialise_ns87308(void);

/*
 * Minimal serial functions needed to use one87308's UARTs
 * as serial console interface.
 */

volatile struct NS16550 *console;

void
serial_init (unsigned long dummy, int baudrate)
{
  init_data_t * const idata = IDATA;
  const int clock_divisor = 115200/baudrate;
  initialise_ns87308();
  idata->console_addr = (void *)NS16550_init(0, clock_divisor);
}

void
serial_putc(const char c)
{
  init_data_t * const idata = IDATA;
  NS16550_putc((struct NS16550 *) idata->console_addr, c);
  if(c == '\n') NS16550_putc((struct NS16550 *) idata->console_addr, '\r');
}

void
serial_puts (const char *s)
{
  while (*s) {
    serial_putc (*s++);
  }
}

int
serial_getc(void)
{
  init_data_t * const idata = IDATA;
  return NS16550_getc((struct NS16550 *) idata->console_addr);
}

int
serial_tstc(void)
{
  init_data_t * const idata = IDATA;
  return NS16550_tstc((struct NS16550 *) idata->console_addr);
}

void
serial_setbrg (unsigned long dummy, int baudrate)
{
  init_data_t * const idata = IDATA;
  const int clock_divisor = 115200/baudrate;
  NS16550_reinit((struct NS16550 *) idata->console_addr, clock_divisor);
}
